Attribute VB_Name = "ExportData"
'Handles the exporting of grid contents and Underlying Queries

Option Explicit

Public Sub ExportGridToExcel(objCallingForm As Form, objBodyGrid As MSFlexGrid, _
                      Optional objHeader As MSFlexGrid, Optional blnHideProgress As Boolean)

    'Exports the contents of the specified grid to an Excel Worksheet
    On Local Error GoTo errorHandler
    
    
    Dim objFileSystem As New FileSystemObject
    
    Dim objWorkbook As Excel.Workbook
    Dim objWorksheet As Excel.Worksheet
    Dim strColLabel As String
    Dim lngOffSet As Long
    
    Dim lngRowCounter As Long
    Dim lngColCounter As Long
    Dim strLine As String
    Dim strTargetFile As String
    Dim lngFileHandle As Long
    Dim strMsg As String
    
    Dim lngNumberProcessed As Long
    Dim lngJobLength As Long
    
    'Get filename
    With frmSystem_MainMenu.mainCommonDialog
        .DialogTitle = "Export Results"
        .Filter = "Excel Files (*.xls)|*.xls"
        .Flags = cdlOFNPathMustExist + cdlOFNOverwritePrompt
        .FileName = ""
        .CancelError = True
        .ShowSave
    End With
    strTargetFile = frmSystem_MainMenu.mainCommonDialog.FileName
    
    'Delete file if present
    'Above dialog will have given an overwrite prompt
    If objFileSystem.FileExists(strTargetFile) Then
        objFileSystem.DeleteFile (strTargetFile)
    End If
    
    Screen.MousePointer = vbHourglass
    
    If Not blnHideProgress Then LoadProgressForm objCallingForm, "Exporting Data To Microsoft Excel"
    
    'Calculate the number of rows to be exported
    lngJobLength = objBodyGrid.Rows + IIf(IsMissing(objHeader), 0, objHeader.Rows)
    
    'Create Workbook
    Set objWorkbook = Excel.Workbooks.Add
    Set objWorksheet = objWorkbook.Worksheets(1)
    objWorkbook.SaveAs strTargetFile
    
    'initialise progress lngCounter
    lngNumberProcessed = 0
    
    'Export Header Grid If Present
    If Not IsMissing(objHeader) Then
        For lngRowCounter = 0 To objHeader.Rows - 1
            For lngColCounter = 0 To objHeader.Cols - 1
                objWorksheet.Cells(lngRowCounter + 1, lngColCounter + 1) _
                          = GetGridData(objHeader, lngRowCounter, lngColCounter)
                objWorksheet.Cells(lngRowCounter + 1, lngColCounter + 1).Font.Bold = True
            Next
            lngNumberProcessed = lngNumberProcessed + 1
            If Not blnHideProgress Then ShowProgress lngNumberProcessed, lngJobLength
        Next
        lngOffSet = objHeader.Rows + 2
    Else
        lngOffSet = 0
    End If
    
    'Export body header
    objBodyGrid.Row = 0
    For lngColCounter = 1 To objBodyGrid.Cols - 1
        objBodyGrid.Col = lngColCounter
        objWorksheet.Cells(1 + lngOffSet, lngColCounter).Value = objBodyGrid.Text
        objWorksheet.Cells(1 + lngOffSet, lngColCounter).Font.Bold = True
    Next
    lngNumberProcessed = lngNumberProcessed + 1
    If Not blnHideProgress Then ShowProgress lngNumberProcessed, lngJobLength
    
    'Export Body
    For lngRowCounter = 1 To objBodyGrid.Rows - 1
        objBodyGrid.Row = lngRowCounter
        For lngColCounter = 1 To objBodyGrid.Cols - 1
            objBodyGrid.Col = lngColCounter
            
            If objBodyGrid.CellForeColor = vbRed Then
                strLine = Str(Val(objBodyGrid.Text) * -1)
            ElseIf objBodyGrid.CellForeColor = vbBlue Then
                strLine = Str(Val(objBodyGrid.Text))
            ElseIf ContainsCarriageReturn(objBodyGrid.Text) Then
                strLine = ReformatMultilineText(objBodyGrid.Text, Chr(10))
                Call setWrapText(lngColCounter, objWorksheet)
            Else
                strLine = objBodyGrid.Text
            End If
            objWorksheet.Cells(lngRowCounter + lngOffSet + 1, lngColCounter).Value = strLine
        Next
        lngNumberProcessed = lngNumberProcessed + 1
        If Not blnHideProgress Then ShowProgress lngNumberProcessed, lngJobLength
    Next
    
    'set vertical alignment
    objWorksheet.UsedRange.VerticalAlignment = xlTop
    
    'Save and release workbook
    objWorkbook.Save
    objWorkbook.Close
    
    UnloadProgressForm

    Screen.MousePointer = vbDefault
    
    'Inform user of successful exportation
    strMsg = "Data successfully exported to " & strTargetFile
    MsgBox strMsg, vbInformation, "Export To Excel"

    
Exit Sub

errorHandler:
    'Display error and abort
    If Err.Number <> cdlCancel Then
        MsgBox Err.Description, vbExclamation, "Aborting"
    End If
    If Not blnHideProgress Then UnloadProgressForm
    Call ForceWorkbookRelease(objWorkbook)
    Screen.MousePointer = vbDefault
    Exit Sub
Resume

End Sub

Private Function setWrapText(lngColNumber As Long, Worksheet As Excel.Worksheet) As String
    Dim strColLabel As String
    
    'Convert the column number into an excel column label
    If 1 <= lngColNumber <= 26 Then
        strColLabel = Chr(Asc("A") + lngColNumber - 1)
    ElseIf 27 <= lngColNumber <= 26 * 25 Then
        strColLabel = Chr(Asc("A") + Int((lngColNumber - 1) / 26) - 1)
        strColLabel = strColLabel + Chr(Asc("A") + ((lngColNumber - 1) Mod 26))
    End If
    strColLabel = strColLabel & ":" & strColLabel
    
    'Change the cell formating so that it accepts multiline text
    If Worksheet.Range(strColLabel).WrapText = False Then
        Worksheet.Range(strColLabel).WrapText = True
        Worksheet.Range(strColLabel).ColumnWidth = 30
    End If
    
End Function

Public Sub ExportGridToGeneric(objCallingForm As Form, objBodyGrid As MSFlexGrid, _
                      Optional objHeader As MSFlexGrid, Optional blnHideProgress As Boolean)
    
    'Exports to a standard comma deliminated text file
    
    Dim lngRowCounter As Long
    Dim lngColCounter As Long
    Dim strLine As String
    Dim strTargetFile As String
    Dim strMsg As String
    
    Dim lngNumberProcessed As Long
    Dim lngJobLength As Long
    
    Dim objFileSystem As New FileSystemObject
    Dim objFile As TextStream
    
    On Local Error GoTo errorHandler
    
    'get file name for saving text file
    With frmSystem_MainMenu.mainCommonDialog
        .DialogTitle = "Export Results"
        .Filter = "Text Files (*.TXT)|*.txt"
        .Flags = cdlOFNPathMustExist + cdlOFNOverwritePrompt
        .FileName = ""
        .CancelError = True
        .ShowSave
    End With
    
    'Save to a standard comma deliminated text file
    strTargetFile = frmSystem_MainMenu.mainCommonDialog.FileName
    Screen.MousePointer = vbHourglass
    
    'initialise progress display
    If Not blnHideProgress Then Call LoadProgressForm(objCallingForm, "Exporting To Generic Format")
    
    lngNumberProcessed = 0
    lngJobLength = objBodyGrid.Rows + IIf(IsMissing(objHeader), 0, objHeader.Rows)
        
    'Open file
    Set objFile = objFileSystem.OpenTextFile(strTargetFile, ForWriting, True)
        'Export Header Grid If Present
        If Not IsMissing(objHeader) Then
            For lngRowCounter = 0 To objHeader.Rows - 1
                objHeader.Row = lngRowCounter
                For lngColCounter = 0 To objHeader.Cols - 1
                    objHeader.Col = lngColCounter
                    strLine = strLine & objHeader.Text & ","
                Next
                objFile.WriteLine strLine
                strLine = ""
                
                'indicate progress
                lngNumberProcessed = lngNumberProcessed + 1
                If Not blnHideProgress Then ShowProgress lngNumberProcessed, lngJobLength
            Next
            
            'write 2 blank lines
            objFile.WriteBlankLines (2)
        End If
        
        'Export body grid header
        objBodyGrid.Row = 0
        
        For lngColCounter = 1 To objBodyGrid.Cols - 1
            objBodyGrid.Col = lngColCounter
            strLine = strLine & objBodyGrid.Text & ","
        Next
        objFile.WriteLine strLine
        lngNumberProcessed = lngNumberProcessed + 1
        If Not blnHideProgress Then ShowProgress lngNumberProcessed, lngJobLength
    
        strLine = ""
        
        For lngRowCounter = 1 To objBodyGrid.Rows - 1
            objBodyGrid.Row = lngRowCounter
            For lngColCounter = 1 To objBodyGrid.Cols - 1
                objBodyGrid.Col = lngColCounter
                
                If objBodyGrid.CellForeColor = vbRed Then
                    strLine = strLine & Str(Val(objBodyGrid.Text) * -1) & ","
                ElseIf objBodyGrid.CellForeColor = vbBlue Then
                    strLine = strLine & Str(Val(objBodyGrid.Text)) & ","
                ElseIf ContainsCarriageReturn(objBodyGrid.Text) Then
                    strLine = strLine & ReformatMultilineText(objBodyGrid.Text, ";")
                Else
                    strLine = strLine & objBodyGrid.Text & ","
                End If
            Next
            objFile.WriteLine strLine
            strLine = ""
            
            'indicate progress
            lngNumberProcessed = lngNumberProcessed + 1
            If Not blnHideProgress Then ShowProgress lngNumberProcessed, lngJobLength
        Next
    objFile.Close
    
    Screen.MousePointer = vbDefault
    
    If Not blnHideProgress Then UnloadProgressForm
    
    'Reload page that was being viewed
    strMsg = "Data successfully exported to " & strTargetFile
    MsgBox strMsg, vbInformation, "Export To Text File"
            
Exit Sub
            
errorHandler:
    If Not blnHideProgress Then UnloadProgressForm
    If Err.Number <> cdlCancel Then
        MsgBox Err.Description, vbExclamation, "Aborting"
    End If

End Sub

Public Function ReformatMultilineText(strMultilineText As String, strSeparator As String) As String
    'Used in exporting to Comma deliminated text file.
    'Takes a piece of text and then replaces all occurances of the carriage return
    'with the supplied separator
    
    Dim strLine As String
    Dim strCurrentChar As String * 1
    Dim lngCharCounter As Long
    Dim lngLineStarts As Long
    Dim lngStringLength As Long
        
    Dim strCarriageReturn As String * 1
    Dim strLineFeed As String * 1
    
    Dim strReformated As String
    
    strCarriageReturn = vbCr
    strLineFeed = vbLf
    
    'Variables default to zero. Reinitialise them to 1 for the MID function
    lngLineStarts = 1
    lngCharCounter = 1
    lngStringLength = Len(strMultilineText)
    
    While (lngCharCounter <= lngStringLength)    'More characters still to be read
        strCurrentChar = Mid(strMultilineText, lngCharCounter, 1)
        If ((strCurrentChar = strCarriageReturn) Or (strCurrentChar = strLineFeed)) Then
            'Rip out line
            strLine = Mid(strMultilineText, lngLineStarts, lngCharCounter - lngLineStarts)
            'Copy it to the formated text
            If strReformated = "" Then
                strReformated = strLine
            Else
                strReformated = strReformated & strSeparator & strLine
            End If
            
            'Skip over any subsequent line feeds or carriage returns
            While (((strCurrentChar = strCarriageReturn) Or (strCurrentChar = strLineFeed)) And lngCharCounter <= Len(strMultilineText))
                lngCharCounter = lngCharCounter + 1
                strCurrentChar = Mid(strMultilineText, lngCharCounter, 1)
            Wend
            'Mark where next line will start
            lngLineStarts = lngCharCounter
        Else
            lngCharCounter = lngCharCounter + 1
        End If
    Wend
    
    'Check for a trailing segment i.e. between last carriage and end of line
    If lngLineStarts <= Len(strMultilineText) Then
        strLine = Mid(strMultilineText, lngLineStarts, lngStringLength - lngLineStarts + 1)
        If strReformated = "" Then
            strReformated = strLine
        Else
            strReformated = strReformated & strSeparator & strLine
        End If
    End If
            
    'Return the formated text
    ReformatMultilineText = strReformated
    
End Function

Private Function ContainsCarriageReturn(strText As String) As Boolean
    'Checks if a piece of text contains a carriage return
    Dim lngCharCounter As Long
    Dim strCharacter As String * 1
    
    For lngCharCounter = 1 To Len(strText)
        strCharacter = Mid(strText, lngCharCounter, 1)
        If strCharacter = vbCr Then
            ContainsCarriageReturn = True
            Exit For
        End If
    Next
    
End Function

Private Sub ForceWorkbookRelease(objObject As Object)
    On Local Error Resume Next
    objObject.Save
    objObject.Close
End Sub

Public Sub ExportSourceSQL(strSQL As String)
    'Saves the strSQL to a standard text file
    
    
    Dim strTargetFile As String
    Dim objFileSystem As New FileSystemObject
    Dim objFile As TextStream
    
    On Local Error GoTo errorHandler
    
    'get file name for saving text file
    With frmSystem_MainMenu.mainCommonDialog
        .DialogTitle = "Export Query Source File"
        .Filter = "SQL Files Files (*.SQL)|*.sql|Text Files (*.TXT)|*.txt"
        .FilterIndex = 1
        .Flags = cdlOFNPathMustExist + cdlOFNOverwritePrompt
        .FileName = ""
        .CancelError = True
        .ShowSave
    End With
    
    'get filename
    strTargetFile = frmSystem_MainMenu.mainCommonDialog.FileName
    Screen.MousePointer = vbHourglass
    
    'Save to file
    Set objFile = objFileSystem.OpenTextFile(strTargetFile, ForWriting, True)
        objFile.Write strSQL
    objFile.Close
    
    Screen.MousePointer = vbDefault
            
Exit Sub
            
errorHandler:
    If Err.Number <> cdlCancel Then
        MsgBox Err.Description, vbExclamation, "Aborting"
    End If
End Sub
