Attribute VB_Name = "GeneralMailMergeRoutines"
Attribute VB_Description = "Report general functions and declarations"
Option Explicit

Public rstQueryResults As Recordset

Public Const strFieldStartChar = ""
Public Const strFieldEndChar = ""
Public blnReplaceFunctionalityLoaded  As Boolean

Public Function Embedded_in(lngStart As Long, strBeingSearched As String, _
                            strSearchString As String, blnWholeWord As Boolean, _
                            blnMatchCase As Boolean) As Integer
    'Finds the first occurance of strSearchString within strBeingSearched starting
    'at lngStart. Also gives the traditional whole word and match case options
    
    Dim lngLocation As Long
    Dim lngStartPosition As Long
    Dim intSearchMode As Integer
    
    'Infoce case matching. This parameter is passed to the VB inSTR function
    If blnMatchCase Then
        intSearchMode = vbBinaryCompare
    Else
        intSearchMode = vbTextCompare
    End If
    
    If Len(strBeingSearched) = 0 Or Len(strSearchString) = 0 Then
        Embedded_in = 0
    Else
        lngStartPosition = lngStart
        Do
            lngLocation = InStr(lngStartPosition, strBeingSearched, _
                                strSearchString, intSearchMode)
            If lngLocation = 0 Then
                'text not found
                Embedded_in = 0
                Exit Do
            Else
                'text found, proceed to check if optional conditions hold
                If blnWholeWord Then
                    'Whole word option chosen; enforce it
                    If isWholeWord(lngLocation, strSearchString, strBeingSearched) Then
                        Embedded_in = lngLocation
                        Exit Do
                    Else
                        'Whole word test failed. proceed with search
                        lngStartPosition = lngStartPosition + 1
                    End If
                Else
                    'Whole word option not chosen
                    Embedded_in = lngLocation
                    Exit Do
                End If
            End If
        Loop
    End If
                
    
End Function


Public Sub Find_next(Optional blnWholeWord As Boolean, Optional blnMatchCase As Boolean, _
                                                       Optional blnResetOp As Boolean)
    'Note that if a reset is requested, the reset will be performed and the
    'proceedure will return without actually carrying out a find next op
    
    Static lngStartPosition As Long
    
    Dim lngLocatedAt As Long
    Dim strBeingSearched As String
    Dim strSearchString As String
    
    
    If blnResetOp Then
        'request to reset proceedure
        lngStartPosition = 1
    Else
        'Necessary as int type defaults to zero. Our string are however 1 based
        If lngStartPosition = 0 Then lngStartPosition = 1
        
        strBeingSearched = frmMailMerge_MasterForm.rtfMailMerge.Text
        strSearchString = frmMailMerge_SearchAndReplace.txtFindText.Text
        lngLocatedAt = Embedded_in(lngStartPosition, strBeingSearched, _
                                   strSearchString, blnWholeWord, blnMatchCase)
        
        If lngLocatedAt = 0 Then
            MsgBox "Match not found", vbExclamation, "Find Text"
            lngStartPosition = 1
        Else
            'Match found
            
            'Set start position of next search
            lngStartPosition = lngLocatedAt + Len(strSearchString)
            
            'Highlight the found text
            frmMailMerge_MasterForm.rtfMailMerge.SelStart = lngLocatedAt - 1
            frmMailMerge_MasterForm.rtfMailMerge.SelLength = Len(strSearchString)
            
            'Return focus to the parent form.
            frmMailMerge_MasterForm.SetFocus
        End If
    End If
            
End Sub

Private Function isWholeWord(lngWordStartsAt, strSearchString As String, _
                            strBeingSearched As String) As Boolean
    'Uses the strAlphaNumeric constant defined below to check if the character occuring
    'before and after the word should be considered to be part of the word or not. To
    'treat any other char as part of this string, add it to the string below
    
    Dim strFirstChar As String * 1
    Dim strLastChar As String * 1
    Const strAlphaNumeric = "0123456789" _
                          & "ABCDEFGHIJKLMNOPQRSTUVWXYZ" _
                          & "abcdefghijklmnopqrstuvwxyz" _
                          & "" _
                          & ""
    
    'Record the characters occuring before and after the word occurance
    strFirstChar = Mid(strBeingSearched, lngWordStartsAt - 1, 1)
    strLastChar = Mid(strBeingSearched, lngWordStartsAt + Len(strSearchString), 1)
    
    'Check if either of the characters appears within the string of chacaters not
    'considered word separators.
    If (InStr(1, strAlphaNumeric, strFirstChar) > 0) _
    Or (InStr(1, strAlphaNumeric, strLastChar) > 0) Then
        isWholeWord = False
    Else
        isWholeWord = True
    End If
    
End Function

Public Sub Add_replace_functionality()
    
    'Takes the Search and replace form and adds the controls required
    'for search amd replace functions.
    
    With frmMailMerge_SearchAndReplace
        .cmdLoadReplace.Visible = False
        .cmdReplace.Visible = True
        .cmdReplaceAll.Visible = True
        .txtReplacement.Visible = True
        .lblReplaceWith.Visible = True
        .cmdReplace.Default = True
                
        .cmdFindNext.Top = 180
        .cmdReplace.Top = 660
        .cmdReplaceAll.Top = 1140
        .cmdHelp.Top = 1620
        .cmdCancel.Top = 2100
        .fraOptions.Top = 1200
        
        .Height = 3015
        .Caption = "Search and Replace"
    End With
    
    'flag to enable help to know the topic to load
    blnReplaceFunctionalityLoaded = True
    
End Sub

Public Sub Remove_replace_functionality()
    
    'Customises the search and replace form so that it excludes the controls required
    'for 'replace' functionality creating a typical Find window
    
    With frmMailMerge_SearchAndReplace
        .cmdLoadReplace.Visible = True
        .cmdReplace.Visible = False
        .cmdReplaceAll.Visible = False
        .txtReplacement.Visible = False
        .lblReplaceWith.Visible = False
        .cmdFindNext.Default = True
                
        .cmdFindNext.Top = 180
        .cmdLoadReplace.Top = 660
        .cmdHelp.Top = 1140
        .cmdCancel.Top = 1620
        .fraOptions.Top = 780
        
        .Height = 2670
        .Caption = "Find"
    End With
    
    blnReplaceFunctionalityLoaded = False
End Sub

